@@ -8,9 +8,8 @@ from scipy import signal
 from scipy.io import wavfile
 
 from ibllib import dsp
-from ibllib.misc import log2session_static
 import ibllib.io.raw_data_loaders as ioraw
-import ibllib.io.extractors.training_trials
+from ibllib.io.extractors.training_trials import GoCueTimes
 
 logger_ = logging.getLogger('ibllib')
 
@@ -99,7 +98,6 @@ def welchogram(fs, wav, nswin=NS_WIN, overlap=OVERLAP, nperseg=NS_WELCH):
     return tscale, fscale, W, detect
 
 
-@log2session_static('extraction')
 def extract_sound(ses_path, save=True, force=False, delete=False):
     """
     Simple audio features extraction for ambient sound characterization.
@@ -107,29 +105,29 @@ def extract_sound(ses_path, save=True, force=False, delete=False):
 
     :param ses_path: ALF full session path: (/mysubject001/YYYY-MM-DD/001)
     :param delete: if True, removes the wav file after processing
-    :return: None
+    :return: list of output files
     """
     ses_path = Path(ses_path)
     wav_file = ses_path / 'raw_behavior_data' / '_iblrig_micData.raw.wav'
     out_folder = ses_path / 'raw_behavior_data'
-    if not wav_file.exists():
-        return None
     files_out = {'power': out_folder / '_iblmic_audioSpectrogram.power.npy',
                  'frequencies': out_folder / '_iblmic_audioSpectrogram.frequencies.npy',
                  'onset_times': out_folder / '_iblmic_audioOnsetGoCue.times_mic.npy',
                  'times_microphone': out_folder / '_iblmic_audioSpectrogram.times_mic.npy',
-                 'times': out_folder / '_iblmic_audioSpectrogram.times.npy'
                  }
     # if they exist and the option Force is set to false, do not recompute and exit
     if all([files_out[f].exists() for f in files_out]) and not force:
-        logger_.warning('Output exists. Skipping ' + str(wav_file) + ' Use force flag to override')
-        return
+        logger_.warning(f'Output exists. Skipping {wav_file}. Use force flag to override')
+        return [files_out[k] for k in files_out]
+    if not wav_file.exists():
+        logger_.warning(f"Wav file doesn't exist: {wav_file}")
+        return []
     # crunch the wav file
     fs, wav = wavfile.read(wav_file, mmap=False)
     if len(wav) == 0:
         status = _fix_wav_file(wav_file)
         if status != 0:
-            logger_.error(f"WAV Header Indicates empty file. Couldn't fix. Abort. {wav_file}")
+            logger_.error(f"WAV Header empty. Sox couldn't fix it, Abort. {wav_file}")
             return
         else:
             fs, wav = wavfile.read(wav_file, mmap=False)
@@ -145,16 +143,17 @@ def extract_sound(ses_path, save=True, force=False, delete=False):
     data = ioraw.load_data(ses_path)
     if data is None:  # if no session data, we're done
         return
-    tgocue = np.array(ibllib.io.extractors.training_trials.get_goCueOnset_times(
-        None, save=False, data=data))
+    tgocue, _ = GoCueTimes(ses_path).extract(save=False, bpod_trials=data)
     ilast = min(len(tgocue), len(detect))
     dt = tgocue[:ilast] - detect[: ilast]
     # only save if dt is consistent for the whole session
     if np.std(dt) < 0.2 and save:
+        files_out['times'] = out_folder / '_iblmic_audioSpectrogram.times.npy'
         tscale += np.median(dt)
         np.save(file=files_out['times'], arr=tscale[:, None].astype(np.single))
     if delete:
         wav_file.unlink()
+    return [files_out[k] for k in files_out]
 
 
 def _fix_wav_file(wav_file):
