@@ -1,37 +1,23 @@
-#!/usr/bin/env python
-# -*- coding:utf-8 -*-
-"""
-Training wheel extractor from Pybpod output.
-"""
-import os
 import logging
+from collections.abc import Sized
 
 import numpy as np
 from scipy import interpolate
 
+from ibllib.io.extractors import training_trials
+from ibllib.io.extractors.base import BaseBpodTrialsExtractor
 import ibllib.io.raw_data_loaders as raw
 from ibllib.misc import structarr
 import ibllib.exceptions as err
+import brainbox.behavior.wheel as wh
 
-logger_ = logging.getLogger('ibllib.alf')
+_logger = logging.getLogger('ibllib')
 WHEEL_RADIUS_CM = 1  # we want the output in radians
 THRESHOLD_RAD_PER_SEC = 10
 THRESHOLD_CONSECUTIVE_SAMPLES = 0
 EPS = 7. / 3 - 4. / 3 - 1
 
 
-def check_alf_folder(session_path):
-    """
-    Check if alf folder exists, creates it if it doesn't.
-
-    :param session_path: absolute path of session folder
-    :type session_path: str
-    """
-    alf_folder = os.path.join(session_path, 'alf')
-    if not os.path.exists(alf_folder):
-        os.mkdir(alf_folder)
-
-
 def get_trial_start_times(session_path, data=None):
     if not data:
         data = raw.load_data(session_path)
@@ -45,8 +31,7 @@ def get_trial_start_times(session_path, data=None):
 def sync_rotary_encoder(session_path, bpod_data=None, re_events=None):
     if not bpod_data:
         bpod_data = raw.load_data(session_path)
-    if not re_events:
-        evt = raw.load_encoder_events(session_path)
+    evt = re_events or raw.load_encoder_events(session_path)
     # we work with stim_on (2) and closed_loop (3) states for the synchronization with bpod
     tre = evt.re_ts.values / 1e6  # convert to seconds
     # the first trial on the rotary encoder is a dud
@@ -63,7 +48,7 @@ def sync_rotary_encoder(session_path, bpod_data=None, re_events=None):
                                          " synchronization. Wheel data not extracted")
     # bpod bug that spits out events in ms instead of us
     if np.diff(bpod['closed_loop'][[-1, 0]])[0] / np.diff(rote['closed_loop'][[-1, 0]])[0] > 900:
-        logger_.error("Rotary encoder stores values in ms instead of us. Wheel timing inaccurate")
+        _logger.error("Rotary encoder stores values in ms instead of us. Wheel timing inaccurate")
         rote['stim_on'] *= 1e3
         rote['closed_loop'] *= 1e3
     # just use the closed loop for synchronization
@@ -101,36 +86,23 @@ def sync_rotary_encoder(session_path, bpod_data=None, re_events=None):
     return interpolate.interp1d(re, bp, fill_value="extrapolate")
 
 
-def get_wheel_position(session_path, bp_data=None, save=False, display=False):
+def get_wheel_position(session_path, bp_data=None, display=False):
     """
-    Get wheel data from raw files and converts positions into radians mathematical convention
-     (anti-clockwise = +) and timestamps into seconds relative to Bpod clock.
-    **Optional:** saves _ibl_wheel.times.npy and _ibl_wheel.position.npy
-
-    Times:
-    Gets Rotary Encoder timestamps (us) for each position and converts to times.
-    Synchronize with Bpod and outputs
-
-    Positions:
-    Radians mathematical convention
-
-    :param session_path: absolute path of session folder
-    :type session_path: str
-    :param data: dictionary containing the contents pybppod jsonable file read with raw.load_data
-    :type data: dict, optional
-    :param save: wether to save the corresponding alf file
-                 to the alf folder, defaults to False
-    :type save: bool, optional
-    :return: Numpy structured array.
-    :rtype: numpy.ndarray
+    Gets wheel timestamps and position. Position is in radian (constant above for radius is 1)
+    mathematical convention.
+    :param session_path:
+    :param bp_data (optional): bpod trials read from jsonable file
+    :param display (optional): (bool)
+    :return: timestamps (np.array)
+    :return: positions (np.array)
     """
     status = 0
     if not bp_data:
         bp_data = raw.load_data(session_path)
     df = raw.load_encoder_positions(session_path)
     if df is None:
-        logger_.error('No wheel data for ' + str(session_path))
-        return None
+        _logger.error('No wheel data for ' + str(session_path))
+        return None, None
     data = structarr(['re_ts', 're_pos', 'bns_ts'],
                      shape=(df.shape[0],), formats=['f8', 'f8', np.object])
     data['re_ts'] = df.re_ts.values
@@ -181,8 +153,8 @@ def get_wheel_position(session_path, bp_data=None, save=False, display=False):
                 # at which point we are running out of possible bugs and calling it
             tr_dc[ind] = data['re_pos'][ind - 1]
         if iwarn:  # if a warning flag was caught in the loop throw a single warning
-            logger_.warning('Rotary encoder reset events discrepancy. Doing my best to merge.')
-            logger_.debug('Offending inds: ' + str(iwarn) + ' times: ' + str(data['re_ts'][iwarn]))
+            _logger.warning('Rotary encoder reset events discrepancy. Doing my best to merge.')
+            _logger.debug('Offending inds: ' + str(iwarn) + ' times: ' + str(data['re_ts'][iwarn]))
         # exit status 0 is fine, 1 something went wrong
         return tr_dc, len(iwarn) != 0
 
@@ -257,16 +229,158 @@ def get_wheel_position(session_path, bp_data=None, save=False, display=False):
         ax.plot(data['re_ts'], data['re_pos'] / WHEEL_RADIUS_CM, '.-', label='Output Trace')
         ax.legend()
         # plt.hist(np.diff(data['re_ts']), 400, range=[0, 0.01])
-
-    check_alf_folder(session_path)
-    if raw.save_bool(save, '_ibl_wheel.timestamps.npy'):
-        tpath = os.path.join(session_path, 'alf', '_ibl_wheel.timestamps.npy')
-        np.save(tpath, data['re_ts'])
-    if raw.save_bool(save, '_ibl_wheel.position.npy'):
-        ppath = os.path.join(session_path, 'alf', '_ibl_wheel.position.npy')
-        np.save(ppath, data['re_pos'])
-    return data
+    return data['re_ts'], data['re_pos']
 
 
-def extract_all(session_path, bp_data=None, save=False):
-    return get_wheel_position(session_path, bp_data=bp_data, save=save)
+def extract_wheel_moves(re_ts, re_pos, display=False):
+    """
+    Extract wheel positions and times from sync fronts dictionary
+    :param re_ts: numpy array of rotary encoder timestamps
+    :param re_pos: numpy array of rotary encoder positions
+    :param display: bool: show the wheel position and velocity for full session with detected
+    movements highlighted
+    :return: wheel_moves dictionary
+    """
+    if len(re_ts.shape) == 1:
+        assert re_ts.size == re_pos.size, 'wheel data dimension mismatch'
+    else:
+        _logger.debug('2D wheel timestamps')
+        if len(re_pos.shape) > 1:  # Ensure 1D array of positions
+            re_pos = re_pos.flatten()
+        # Linearly interpolate the times
+        x = np.arange(re_pos.size)
+        re_ts = np.interp(x, re_ts[:, 0], re_ts[:, 1])
+
+    # Check the values and units of wheel position
+    res = np.array([wh.ENC_RES, wh.ENC_RES / 2, wh.ENC_RES / 4])
+    # min change in rad and cm for each decoding type
+    # [rad_X4, rad_X2, rad_X1, cm_X4, cm_X2, cm_X1]
+    min_change = np.concatenate([2 * np.pi / res, wh.WHEEL_DIAMETER * np.pi / res])
+    pos_diff = np.median(np.abs(np.ediff1d(re_pos)))
+
+    # find min change closest to min pos_diff
+    idx = np.argmin(np.abs(min_change - pos_diff))
+    if idx < len(res):
+        # Assume values are in radians
+        units = 'rad'
+        encoding = idx
+    else:
+        units = 'cm'
+        encoding = idx - len(res)
+    enc_names = {0: 'X4', 1: 'X2', 2: 'X1'}
+    _logger.info('Wheel in %s units using %s encoding', units, enc_names[int(encoding)])
+
+    # The below assertion is violated by Bpod wheel data
+    #  assert np.allclose(pos_diff, min_change, rtol=1e-05), 'wheel position skips'
+
+    # Convert the pos threshold defaults from samples to correct unit
+    thresholds = wh.samples_to_cm(np.array([8, 1.5]), resolution=res[encoding])
+    if units == 'rad':
+        thresholds = wh.cm_to_rad(thresholds)
+    kwargs = {'pos_thresh': thresholds[0],
+              'pos_thresh_onset': thresholds[1],
+              'make_plots': display}
+
+    # Interpolate and get onsets
+    pos, t = wh.interpolate_position(re_ts, re_pos, freq=1000)
+    on, off, amp, peak_vel = wh.movements(t, pos, freq=1000, **kwargs)
+    assert on.size == off.size, 'onset/offset number mismatch'
+    assert np.all(np.diff(on) > 0) and np.all(
+        np.diff(off) > 0), 'onsets/offsets not strictly increasing'
+    assert np.all((off - on) > 0), 'not all offsets occur after onset'
+
+    # Put into dict
+    wheel_moves = {
+        'intervals': np.c_[on, off], 'peakAmplitude': amp, 'peakVelocity_times': peak_vel}
+    return wheel_moves
+
+
+def extract_first_movement_times(wheel_moves, trials, min_qt=None):
+    """
+    Extracts the time of the first sufficiently large wheel movement for each trial.
+    To be counted, the movement must occur between go cue / stim on and before feedback /
+    response time.  The movement onset is sometimes just before the cue (occurring in the
+    gap between quiescence end and cue start, or during the quiescence period but sub-
+    threshold).  The movement is sufficiently large if it is greater than or equal to THRESH
+    :param wheel_moves: dictionary of detected wheel movement onsets and peak amplitudes for
+    use in extracting each trial's time of first movement.
+    :param trials: dictionary of trial data
+    :param min_qt: the minimum quiescence period, if None a default is used
+    :return: numpy array of first movement times, bool array indicating whether movement
+    crossed response threshold, and array of indices for wheel_moves arrays
+    """
+    THRESH = .1  # peak amp should be at least .1 rad; ~1/3rd of the distance to threshold
+    MIN_QT = .2  # default minimum enforced quiescence period
+
+    # Determine minimum quiescent period
+    if min_qt is None:
+        min_qt = MIN_QT
+        _logger.info('minimum quiescent period assumed to be %.0fms', MIN_QT * 1e3)
+    elif isinstance(min_qt, Sized) and len(min_qt) > len(trials['goCue_times']):
+        min_qt = np.array(min_qt[0:trials['goCue_times'].size])
+
+    # Initialize as nans
+    first_move_onsets = np.full(trials['goCue_times'].shape, np.nan)
+    ids = np.full(trials['goCue_times'].shape, int(-1))
+    is_final_movement = np.zeros(trials['goCue_times'].shape, bool)
+    flinch = abs(wheel_moves['peakAmplitude']) < THRESH
+    all_move_onsets = wheel_moves['intervals'][:, 0]
+    # Iterate over trials, extracting onsets approx. within closed-loop period
+    cwarn = 0
+    for i, (t1, t2) in enumerate(zip(trials['goCue_times'] - min_qt,
+                                     trials['feedback_times'])):
+        if ~np.isnan(t2 - t1):  # If both timestamps defined
+            mask = (all_move_onsets > t1) & (all_move_onsets < t2)
+            if np.any(mask):  # If any onsets for this trial
+                trial_onset_ids, = np.where(mask)
+                if np.any(~flinch[mask]):  # If any trial moves were sufficiently large
+                    ids[i] = trial_onset_ids[~flinch[mask]][0]  # Find first large move id
+                    first_move_onsets[i] = all_move_onsets[ids[i]]  # Save first large onset
+                    is_final_movement[i] = ids[i] == trial_onset_ids[-1]  # Final move of trial
+        else:  # Log missing timestamps
+            cwarn += 1
+    if cwarn:
+        _logger.warning(f'no reliable goCue/Feedback times (both needed) for {cwarn} trials')
+
+    return first_move_onsets, is_final_movement, ids[ids != -1]
+
+
+class Wheel(BaseBpodTrialsExtractor):
+    """
+    Get wheel data from raw files and converts positions into radians mathematical convention
+     (anti-clockwise = +) and timestamps into seconds relative to Bpod clock.
+    **Optional:** saves _ibl_wheel.times.npy and _ibl_wheel.position.npy
+
+    Times:
+    Gets Rotary Encoder timestamps (us) for each position and converts to times.
+    Synchronize with Bpod and outputs
+
+    Positions:
+    Radians mathematical convention
+    """
+    save_names = ('_ibl_wheel.timestamps.npy', '_ibl_wheel.position.npy',
+                  '_ibl_wheelMoves.intervals.npy', '_ibl_wheelMoves.peakAmplitude.npy',
+                  '_ibl_trials.firstMovement_times.npy')
+    var_names = ('wheel_timestamps', 'wheel_position', 'wheel_moves_intervals',
+                 'wheel_moves_peak_amplitude', 'firstMovement_times')
+
+    def _extract(self):
+        ts, pos = get_wheel_position(self.session_path, self.bpod_trials)
+        moves = extract_wheel_moves(ts, pos)
+
+        # need some trial based info to output the first movement times
+        goCue_times, _ = training_trials.GoCueTimes(self.session_path).extract(
+            save=False, bpod_trials=self.bpod_trials, settings=self.settings)
+        feedback_times, _ = training_trials.FeedbackTimes(self.session_path).extract(
+            save=False, bpod_trials=self.bpod_trials, settings=self.settings)
+        trials = {'goCue_times': goCue_times, 'feedback_times': feedback_times}
+        min_qt = self.settings.get('QUIESCENT_PERIOD', None)
+
+        first_moves, *_ = extract_first_movement_times(moves, trials, min_qt=min_qt)
+
+        return ts, pos, moves['intervals'], moves['peakAmplitude'], first_moves
+
+
+def extract_all(session_path, bpod_trials=None, settings=None, save=False):
+    return Wheel(session_path=session_path).extract(
+        save=save, bpod_trials=bpod_trials, settings=settings)
