@@ -8,18 +8,19 @@ Raw Data Loader functions for PyBpod rig
 Module contains one loader function per raw datafile
 """
 import json
-import wave
 import logging
+import wave
+from datetime import datetime
 from pathlib import Path
+
 import numpy as np
 import pandas as pd
-from datetime import datetime
 
+from alf.io import get_session_path
 from ibllib.io import jsonable
 from ibllib.misc import version
 
-
-logger_ = logging.getLogger('ibllib')
+_logger = logging.getLogger('ibllib')
 
 
 def trial_times_to_times(raw_trial):
@@ -86,10 +87,12 @@ def load_data(session_path, time='absolute'):
     :rtype: list of dicts
     """
     if session_path is None:
+        _logger.warning("No data loaded: session_path is None")
         return
     path = Path(session_path).joinpath("raw_behavior_data")
     path = next(path.glob("_iblrig_taskData.raw*.jsonable"), None)
     if not path:
+        _logger.warning("No data loaded: could not find raw data file")
         return None
     data = jsonable.read(path)
     if time == 'absolute':
@@ -109,10 +112,12 @@ def load_settings(session_path):
     :rtype: dict
     """
     if session_path is None:
+        _logger.warning("No data loaded: session_path is None")
         return
     path = Path(session_path).joinpath("raw_behavior_data")
     path = next(path.glob("_iblrig_taskSettings.raw*.json"), None)
     if not path:
+        _logger.warning("No data loaded: could not find raw settings file")
         return None
     with open(path, 'r') as f:
         settings = json.load(f)
@@ -178,7 +183,7 @@ def load_encoder_events(session_path, settings=False):
 def _load_encoder_ssv_file(file_path, **kwargs):
     file_path = Path(file_path)
     if file_path.stat().st_size == 0:
-        logger_.error(f"{file_path.name} is an empty file. ")
+        _logger.error(f"{file_path.name} is an empty file. ")
         raise ValueError(f"{file_path.name} is an empty file. ABORT EXTRACTION. ")
     return pd.read_csv(file_path, sep=' ', header=None, error_bad_lines=False, **kwargs)
 
@@ -272,6 +277,7 @@ def load_encoder_positions(session_path, settings=False):
         if line.startswith('Position'):
             settings = {'IBLRIG_VERSION_TAG': '0.0.0'}
     if not path:
+        _logger.warning("No data loaded: could not find raw encoderPositions file")
         return None
     if version.ge(settings['IBLRIG_VERSION_TAG'], '5.0.0'):
         return _load_encoder_positions_file_ge5(path)
@@ -377,7 +383,7 @@ def load_mic(session_path):
 
 def _clean_wheel_dataframe(data, label, path):
     if np.any(data.isna()):
-        logger_.warning(label + ' has missing/incomplete records \n %s', path)
+        _logger.warning(label + ' has missing/incomplete records \n %s', path)
     # first step is to re-interpret as numeric objects if not already done
     for col in data.columns:
         if data[col].dtype == np.object and col not in ['bns_ts']:
@@ -395,7 +401,7 @@ def _clean_wheel_dataframe(data, label, path):
             # the first sample may be corrupt, in this case throw away
             if i <= 1:
                 drop_first = i
-                logger_.warning(label + ' rotary encoder positions timestamps'
+                _logger.warning(label + ' rotary encoder positions timestamps'
                                         ' first sample corrupt ' + str(path))
             # if it's an uint32 wraparound, the diff should be close to 2 ** 32
             elif 32 - np.log2(data['re_ts'][i] - data['re_ts'][i + 1]) < 0.2:
@@ -403,13 +409,13 @@ def _clean_wheel_dataframe(data, label, path):
             # there is also the case where 2 positions are swapped and need to be swapped back
 
             elif data['re_ts'][i] > data['re_ts'][i + 1] > data['re_ts'][i - 1]:
-                logger_.warning(label + ' rotary encoder timestamps swapped at index: ' +
+                _logger.warning(label + ' rotary encoder timestamps swapped at index: ' +
                                 str(i) + '  ' + str(path))
                 a, b = data.iloc[i].copy(), data.iloc[i + 1].copy()
                 data.iloc[i], data.iloc[i + 1] = b, a
             # if none of those 3 cases apply, raise an error
             else:
-                logger_.error(label + ' Rotary encoder timestamps are not sorted.' + str(path))
+                _logger.error(label + ' Rotary encoder timestamps are not sorted.' + str(path))
                 data.sort_values('re_ts', inplace=True)
                 data.reset_index(inplace=True)
     if drop_first is not False:
@@ -430,7 +436,7 @@ def _groom_wheel_data_lt5(data, label='file ', path=''):
     sess_len_sec = (datetime.strptime(data['bns_ts'].iloc[-1][:25], '%Y-%m-%dT%H:%M:%S.%f') -
                     datetime.strptime(data['bns_ts'].iloc[0][:25], '%Y-%m-%dT%H:%M:%S.%f')).seconds
     if data['re_ts'].iloc[-1] / (sess_len_sec + 1e-6) < 1e5:  # should be 1e6 normally
-        logger_.warning('Rotary encoder reset logs events in ms instead of us: ' +
+        _logger.warning('Rotary encoder reset logs events in ms instead of us: ' +
                         'RE firmware needs upgrading and wheel velocity is potentially inaccurate')
         data['re_ts'] = data['re_ts'] * 1000
     return data
@@ -445,32 +451,19 @@ def _groom_wheel_data_ge5(data, label='file ', path=''):
     data = _clean_wheel_dataframe(data, label, path)
     # check if the time scale is in ms
     if (data['re_ts'].iloc[-1] - data['re_ts'].iloc[0]) / 1e6 < 20:
-        logger_.warning('Rotary encoder reset logs events in ms instead of us: ' +
+        _logger.warning('Rotary encoder reset logs events in ms instead of us: ' +
                         'RE firmware needs upgrading and wheel velocity is potentially inaccurate')
         data['re_ts'] = data['re_ts'] * 1000
     return data
 
 
 def save_bool(save, dataset_type):
-    logger = logging.getLogger('ibllib.alf')
     if isinstance(save, bool):
         out = save
     elif isinstance(save, list):
         out = (dataset_type in save) or (Path(dataset_type).stem in save)
     if out:
-        logger.debug('extracting' + dataset_type)
-    return out
-
-
-def get_port_events(events: dict, name: str = '') -> list:
-    """Return all timestamps from bpod raw data
-    trial['behavior_data']['Events timestamps'] that match name"""
-    out: list = []
-    for k in events:
-        if name in k:
-            out.extend(events[k])
-    out = sorted(out)
-
+        _logger.debug('extracting' + dataset_type)
     return out
 
 
@@ -479,7 +472,7 @@ def sync_trials_robust(t0, t1, diff_threshold=0.001, drift_threshold_ppm=200, ma
     """
     Attempts to find matching timestamps in 2 time-series that have an offset, are drifting,
     and are most likely incomplete: sizes don't have to match, some pulses may be missing
-    in any serie.
+    in any series.
     Only works with irregular time series as it relies on the derivative to match sync.
     :param t0:
     :param t1:
@@ -519,3 +512,151 @@ def sync_trials_robust(t0, t1, diff_threshold=0.001, drift_threshold_ppm=200, ma
         return t0[ind0], t1[ind1], ind0, ind1
     else:
         return t0[ind0], t1[ind1]
+
+
+def get_task_extractor_type(task_name):
+    """
+    Splits the task name according to naming convention:
+    -   ignores everything
+    _iblrig_tasks_biasedChoiceWorld3.7.0 returns "biased"
+    _iblrig_tasks_trainingChoiceWorld3.6.0 returns "training'
+    :param task_name:
+    :return: one of ['biased', 'habituation', 'training', 'ephys', 'mock_ephys', 'sync_ephys']
+    """
+    if isinstance(task_name, Path):
+        try:
+            settings = load_settings(get_session_path(task_name))
+        except json.decoder.JSONDecodeError:
+            return
+        if settings:
+            task_name = settings.get('PYBPOD_PROTOCOL', None)
+        else:
+            return
+    if '_biasedChoiceWorld' in task_name:
+        return 'biased'
+    elif 'biasedScanningChoiceWorld' in task_name:
+        return 'biased'
+    elif 'biasedVisOffChoiceWorld' in task_name:
+        return 'biased'
+    elif '_habituationChoiceWorld' in task_name:
+        return 'habituation'
+    elif '_trainingChoiceWorld' in task_name:
+        return 'training'
+    elif 'ephysChoiceWorld' in task_name:
+        return 'ephys'
+    elif 'ephysMockChoiceWorld' in task_name:
+        return 'mock_ephys'
+    elif task_name and task_name.startswith('_iblrig_tasks_ephys_certification'):
+        return 'sync_ephys'
+
+
+def get_session_extractor_type(session_path):
+    """
+    From a session path, loads the settings file, finds the task and checks if extractors exist
+    task names examples:
+    :param session_path:
+    :return: bool
+    """
+    settings = load_settings(session_path)
+    if settings is None:
+        _logger.error(f'ABORT: No data found in "raw_behavior_data" folder {session_path}')
+        return False
+    extractor_type = get_task_extractor_type(settings['PYBPOD_PROTOCOL'])
+    if extractor_type:
+        return extractor_type
+    else:
+        _logger.warning(str(session_path) +
+                        f" No extractors were found for {extractor_type} ChoiceWorld")
+        return False
+
+
+def load_bpod_fronts(session_path: str, data: dict = False) -> list:
+    """load_bpod_fronts
+    Loads BNC1 and BNC2 bpod channels times and polarities from session_path
+
+    :param session_path: a valid session_path
+    :type session_path: str
+    :param data: pre-loaded raw data dict, defaults to False
+    :type data: dict, optional
+    :return: List of dicts BNC1 and BNC2 {"times": np.array, "polarities":np.array}
+    :rtype: list
+    """
+    if not data:
+        data = load_data(session_path)
+
+    BNC1_fronts = np.array([[np.nan, np.nan]])
+    BNC2_fronts = np.array([[np.nan, np.nan]])
+    for tr in data:
+        BNC1_fronts = np.append(
+            BNC1_fronts,
+            np.array(
+                [
+                    [x, 1]
+                    for x in tr["behavior_data"]["Events timestamps"].get("BNC1High", [np.nan])
+                ]
+            ),
+            axis=0,
+        )
+        BNC1_fronts = np.append(
+            BNC1_fronts,
+            np.array(
+                [
+                    [x, -1]
+                    for x in tr["behavior_data"]["Events timestamps"].get("BNC1Low", [np.nan])
+                ]
+            ),
+            axis=0,
+        )
+        BNC2_fronts = np.append(
+            BNC2_fronts,
+            np.array(
+                [
+                    [x, 1]
+                    for x in tr["behavior_data"]["Events timestamps"].get("BNC2High", [np.nan])
+                ]
+            ),
+            axis=0,
+        )
+        BNC2_fronts = np.append(
+            BNC2_fronts,
+            np.array(
+                [
+                    [x, -1]
+                    for x in tr["behavior_data"]["Events timestamps"].get("BNC2Low", [np.nan])
+                ]
+            ),
+            axis=0,
+        )
+
+    BNC1_fronts = BNC1_fronts[1:, :]
+    BNC1_fronts = BNC1_fronts[BNC1_fronts[:, 0].argsort()]
+    BNC2_fronts = BNC2_fronts[1:, :]
+    BNC2_fronts = BNC2_fronts[BNC2_fronts[:, 0].argsort()]
+
+    BNC1 = {"times": BNC1_fronts[:, 0], "polarities": BNC1_fronts[:, 1]}
+    BNC2 = {"times": BNC2_fronts[:, 0], "polarities": BNC2_fronts[:, 1]}
+
+    return [BNC1, BNC2]
+
+
+def get_port_events(trial: dict, name: str = '') -> list:
+    """get_port_events
+    Return all event timestamps from bpod raw data trial that match 'name'
+    --> looks in trial['behavior_data']['Events timestamps']
+
+    :param trial: raw trial dict
+    :type trial: dict
+    :param name: name of event, defaults to ''
+    :type name: str, optional
+    :return: Sorted list of event timestamps
+    :rtype: list
+    TODO: add polarities?
+    """
+    out: list = []
+    events = trial['behavior_data']['Events timestamps']
+    for k in events:
+        if name in k:
+            out.extend(events[k])
+    out = sorted(out)
+
+    return out
