@@ -37,19 +37,37 @@ class Reader:
         self.channel_conversion_sample2v = _conversion_sample2v_from_meta(self.meta)
         # if we are not looking at a compressed file, use a memmap, otherwise instantiate mtscomp
         if self.is_mtscomp:
-            self.data = mtscomp.Reader()
-            self.data.open(self.file_bin, self.file_bin.with_suffix('.ch'))
+            self._raw = mtscomp.Reader()
+            self._raw.open(self.file_bin, self.file_bin.with_suffix('.ch'))
         else:
             if self.nc * self.ns * 2 != self.nbytes:
-                _logger.warning(str(sglx_file) + " : meta data and filesize do not checkout")
-            self.data = np.memmap(sglx_file, dtype='int16', mode='r', shape=(self.ns, self.nc))
+                ftsec = self.file_bin.stat().st_size / 2 / self.nc / self.fs
+                _logger.warning(f"{sglx_file} : meta data and filesize do not checkout\n"
+                                f"File size: expected {self.meta['fileSizeBytes']},"
+                                f" actual {self.file_bin.stat().st_size}\n"
+                                f"File duration: expected {self.meta['fileTimeSecs']},"
+                                f" actual {ftsec}\n"
+                                f"Will attempt to fudge the meta-data information.")
+                self.meta['fileTimeSecs'] = ftsec
+            self._raw = np.memmap(sglx_file, dtype='int16', mode='r', shape=(self.ns, self.nc))
+
+    def __getitem__(self, item):
+        if isinstance(item, int) or isinstance(item, slice):
+            return self.read(nsel=item, sync=False)
+        elif len(item) == 2:
+            return self.read(nsel=item[0], csel=item[1], sync=False)
+
+    @property
+    def shape(self):
+        return self.ns, self.nc
 
     @property
     def is_mtscomp(self):
-        return 'cbin' in self.file_bin.suffix and self.file_bin.with_suffix('.ch').exists()
+        return 'cbin' in self.file_bin.suffix
 
     @property
     def version(self):
+        """:return: """
         if not self.meta:
             return None
         return _get_neuropixel_version_from_meta(self.meta)
@@ -89,7 +107,7 @@ class Reader:
         :param slice_c: slice or channel indices
         :return: float32 array
         """
-        darray = np.float32(self.data[nsel, csel])
+        darray = np.float32(self._raw[nsel, csel])
         darray *= self.channel_conversion_sample2v[self.type][csel]
         if sync:
             return darray, self.read_sync(nsel)
@@ -118,7 +136,7 @@ class Reader:
         """
         if not self.meta:
             _logger.warning('Sync trace not labeled in metadata. Assuming last trace')
-        return split_sync(self.data[_slice, _get_sync_trace_indices_from_meta(self.meta)])
+        return split_sync(self._raw[_slice, _get_sync_trace_indices_from_meta(self.meta)])
 
     def read_sync_analog(self, _slice=slice(0, 10000)):
         """
@@ -175,18 +193,20 @@ class Reader:
     def decompress_file(self, keep_original=True, **kwargs):
         """
         Decompresses a mtscomp file
-        :param keep_original: defaults True. If False, the original compressed file is deleted
-         and the current spikeglx.Reader object is modified in place
+        :param keep_original: defaults True. If False, the original compressed file (input)
+        is deleted and the current spikeglx.Reader object is modified in place
+        NB: This is not equivalent to overwrite (which replaces the output file)
         :return: pathlib.Path of the decompressed *.bin file
         """
-        file_out = self.file_bin.with_suffix('.bin')
+        if 'out' not in kwargs:
+            kwargs['out'] = self.file_bin.with_suffix('.bin')
         assert self.is_mtscomp
-        mtscomp.decompress(self.file_bin, self.file_bin.with_suffix('.ch'), out=file_out, **kwargs)
+        mtscomp.decompress(self.file_bin, self.file_bin.with_suffix('.ch'), **kwargs)
         if not keep_original:
             self.file_bin.unlink()
             self.file_bin.with_suffix('.ch').unlink()
-            self.file_bin = file_out
-        return file_out
+            self.file_bin = kwargs['out']
+        return kwargs['out']
 
     def verify_hash(self):
         """
@@ -194,19 +214,24 @@ class Reader:
         :return: boolean
         """
         if self.is_mtscomp:
-            _logger.warning("SHA1 hash is not implemented for compressed ephys. To check "
-                            "the spikeglx acquisition hash, uncompress the file first !")
-            return True
+            with open(self.file_bin.with_suffix('.ch')) as fid:
+                mtscomp_params = json.load(fid)
+            sm = mtscomp_params.get('sha1_compressed', None)
+            if sm is None:
+                _logger.warning("SHA1 hash is not implemented for compressed ephys. To check "
+                                "the spikeglx acquisition hash, uncompress the file first !")
+                return True
+            sm = sm.upper()
         else:
             sm = self.meta.fileSHA1
-            sc = hashfile.sha1(self.file_bin).upper()
-            if sm == sc:
-                log_func = _logger.info
-            else:
-                log_func = _logger.error
-            log_func(f"SHA1 metadata: {sm}")
-            log_func(f"SHA1 computed: {sc}")
-            return sm == sc
+        sc = hashfile.sha1(self.file_bin).upper()
+        if sm == sc:
+            log_func = _logger.info
+        else:
+            log_func = _logger.error
+        log_func(f"SHA1 metadata: {sm}")
+        log_func(f"SHA1 computed: {sc}")
+        return sm == sc
 
 
 def read(sglx_file, first_sample=0, last_sample=10000):
@@ -369,8 +394,10 @@ def _conversion_sample2v_from_meta(meta_data):
     # interprets the gain value from the metadata header:
     if 'imroTbl' in meta_data.keys():  # binary from the probes: ap or lf
         sy_gain = np.ones(int(meta_data['snsApLfSy'][-1]), dtype=np.float32)
+        # imroTbl has 384 entries regardless of no of channels saved, so need to index by n_ch
+        n_chn = _get_nchannels_from_meta(meta_data) - 1
         # the sync traces are not included in the gain values, so are included for broadcast ops
-        gain = re.findall(r'([0-9]* [0-9]* [0-9]* [0-9]* [0-9]*)', meta_data['imroTbl'])
+        gain = re.findall(r'([0-9]* [0-9]* [0-9]* [0-9]* [0-9]*)', meta_data['imroTbl'])[:n_chn]
         out = {'lf': np.hstack((np.array([1 / np.float32(g.split(' ')[-1]) for g in gain]) *
                                 int2volt, sy_gain)),
                'ap': np.hstack((np.array([1 / np.float32(g.split(' ')[-2]) for g in gain]) *
@@ -387,7 +414,7 @@ def _conversion_sample2v_from_meta(meta_data):
 
 def split_sync(sync_tr):
     """
-    The synchronization channelx are stored as single bits, this will split the int16 original
+    The synchronization channels are stored as single bits, this will split the int16 original
     channel into 16 single bits channels
 
     :param sync_tr: numpy vector: samples of synchronisation trace
@@ -417,17 +444,17 @@ def glob_ephys_files(session_path, suffix='.meta', recursive=True, bin_exists=Tr
     Associated to the subfolders where they are
     the expected folder tree is:
     ├── 3A
-    │   ├── imec0
-    │   │   ├── sync_testing_g0_t0.imec0.ap.bin
-    │   │   └── sync_testing_g0_t0.imec0.lf.bin
-    │   └── imec1
-    │       ├── sync_testing_g0_t0.imec1.ap.bin
-    │       └── sync_testing_g0_t0.imec1.lf.bin
+    │        ├── imec0
+    │                ├── sync_testing_g0_t0.imec0.ap.bin
+    │        │        └── sync_testing_g0_t0.imec0.lf.bin
+    │        └── imec1
+    │            ├── sync_testing_g0_t0.imec1.ap.bin
+    │            └── sync_testing_g0_t0.imec1.lf.bin
     └── 3B
         ├── sync_testing_g0_t0.nidq.bin
         ├── imec0
-        │   ├── sync_testing_g0_t0.imec0.ap.bin
-        │   └── sync_testing_g0_t0.imec0.lf.bin
+        │        ├── sync_testing_g0_t0.imec0.ap.bin
+        │        └── sync_testing_g0_t0.imec0.lf.bin
         └── imec1
             ├── sync_testing_g0_t0.imec1.ap.bin
             └── sync_testing_g0_t0.imec1.lf.bin
@@ -474,7 +501,7 @@ def glob_ephys_files(session_path, suffix='.meta', recursive=True, bin_exists=Tr
 
 
 def _mock_spikeglx_file(mock_bin_file, meta_file, ns, nc, sync_depth,
-                        random=False, int2volts=0.6 / 32768):
+                        random=False, int2volts=0.6 / 32768, corrupt=False):
     """
     For testing purposes, create a binary file with sync pulses to test reading and extraction
     """
@@ -492,7 +519,10 @@ def _mock_spikeglx_file(mock_bin_file, meta_file, ns, nc, sync_depth,
         if line.startswith('fileSizeBytes'):
             line = f'fileSizeBytes={ns * nc * 2}\n'
         if line.startswith('fileTimeSecs'):
-            line = f'fileTimeSecs={ns / fs}\n'
+            if corrupt:
+                line = f'fileTimeSecs={ns / fs + 1.8324}\n'
+            else:
+                line = f'fileTimeSecs={ns / fs}\n'
         fid_target.write(line)
     fid_source.close()
     fid_target.close()
