@@ -36,7 +36,7 @@ def behavior_exists(session_path: str) -> bool:  # testOK
     return False
 
 
-def check_transfer(src_session_path: str or Path, dst_session_path: str or Path):
+def check_transfer(src_session_path: str, dst_session_path: str):
     # missing fail test
     src_files = sorted([x for x in Path(src_session_path).rglob('*') if x.is_file()])
     dst_files = sorted([x for x in Path(dst_session_path).rglob('*') if x.is_file()])
@@ -45,7 +45,6 @@ def check_transfer(src_session_path: str or Path, dst_session_path: str or Path)
     for s, d in zip(src_files, dst_files):
         assert s.name == d.name, assert_msg
         assert s.stat().st_size == d.stat().st_size, assert_msg
-    return
 
 
 def rename_session(session_path: str) -> Path:
@@ -216,6 +215,8 @@ def confirm_video_remote_folder(local_folder=False, remote_folder=False, force=F
             remote_session_path / 'raw_video_data',
             force=force)
         flag_file.unlink()
+        create_video_transfer_done_flag(remote_session_path)
+        check_create_raw_session_flag(remote_session_path)
 
 
 def confirm_ephys_remote_folder(local_folder=False, remote_folder=False,
@@ -249,7 +250,11 @@ def confirm_ephys_remote_folder(local_folder=False, remote_folder=False,
         move_ephys_files(str(session_path))
         # Copy wiring files
         copy_wiring_files(str(session_path), iblscripts_folder)
-        create_alyx_probe_insertions(str(session_path))
+        try:
+            create_alyx_probe_insertions(str(session_path))
+        except BaseException as e:
+            print(e, "\nCreation failed, please create the probe insertions manually.",
+                  "Continuing transfer...")
         msg = f"Transfer to {remote_folder} with the same name?"
         resp = input(msg + "\n[y]es/[r]ename/[s]kip/[e]xit\n ^\n> ") or 'y'
         resp = resp.lower()
@@ -271,6 +276,8 @@ def confirm_ephys_remote_folder(local_folder=False, remote_folder=False,
         if not behavior_exists(remote_session_path):
             print(f"No behavior folder found in {remote_session_path}: skipping session...")
             return
+        # TODO: Check flagfiles on src.and dst + alf dir in session folder then remove
+        # Try catch? wher catch condition is force transfer maybe
         transfer_folder(
             session_path / 'raw_ephys_data',
             remote_session_path / 'raw_ephys_data',
@@ -281,7 +288,8 @@ def confirm_ephys_remote_folder(local_folder=False, remote_folder=False,
         if (remote_session_path / 'extract_me.flag').exists():
             (remote_session_path / 'extract_me.flag').unlink()
         # Create remote flags
-        create_ephys_flags(remote_session_path)
+        create_ephys_transfer_done_flag(remote_session_path)
+        check_create_raw_session_flag(remote_session_path)
 
 
 def create_alyx_probe_insertions(session_path: str,
@@ -323,14 +331,14 @@ def create_alyx_probe_insertions(session_path: str,
         if len(alyx_insertion) == 0:
             alyx_insertion = one.alyx.rest('insertions', 'create', data=insdict)
         else:
-            eid = alyx_insertion[0]['url'][-36:]
+            iid = alyx_insertion[0]['id']
             if force:
-                alyx_insertion = one.alyx.rest('insertions', 'update', id=eid, data=insdict)
+                alyx_insertion = one.alyx.rest('insertions', 'update', id=iid, data=insdict)
             else:
                 alyx_insertion = alyx_insertion[0]
 
 
-def create_ephys_flags(session_folder: str or Path):
+def create_ephys_flags(session_folder: str):
     session_path = Path(session_folder)
     flags.write_flag_file(session_path.joinpath('extract_ephys.flag'))
     flags.write_flag_file(session_path.joinpath('raw_ephys_qc.flag'))
@@ -340,6 +348,26 @@ def create_ephys_flags(session_folder: str or Path):
     flags.write_flag_file(probe01_path.joinpath('spike_sorting.flag'))
 
 
+def create_ephys_transfer_done_flag(session_folder: str) -> None:
+    session_path = Path(session_folder)
+    flags.write_flag_file(session_path.joinpath('ephys_data_transferred.flag'))
+
+
+def create_video_transfer_done_flag(session_folder: str) -> None:
+    session_path = Path(session_folder)
+    flags.write_flag_file(session_path.joinpath('video_data_transferred.flag'))
+
+
+def check_create_raw_session_flag(session_folder: str) -> None:
+    session_path = Path(session_folder)
+    ephys = session_path.joinpath('ephys_data_transferred.flag')
+    video = session_path.joinpath('video_data_transferred.flag')
+    if ephys.exists() and video.exists():
+        flags.write_flag_file(session_path.joinpath('raw_session.flag'))
+        ephys.unlink()
+        video.unlink()
+
+
 def rename_ephys_files(session_folder: str) -> None:
     """rename_ephys_files is system agnostic (3A, 3B1, 3B2).
     Renames all ephys files to Alyx compatible filenames. Uses get_new_filename.
@@ -385,7 +413,8 @@ def get_new_filename(filename: str) -> str:  # testOK
                 gt = f'_g{g}_t{t}'
     else:
         if gt is None:
-            raise(NotImplementedError)
+            raise NotImplementedError(f"Can not parse {filename} as an ephys file with gX_tY where"
+                                      f"XY are single digits")
 
     # ext = 'bin' or 'meta'
     if '.bin' in filename:
