@@ -2,6 +2,7 @@ from functools import singledispatch
 from dataclasses import dataclass, field, fields
 from pathlib import Path
 
+from brainbox.io import parquet
 from ibllib.misc import flatten
 
 
@@ -64,9 +65,9 @@ class SessionDataInfo:
             dataset_type=[d['dataset_type'] for d in dsets],
             dataset_id=[d['id'] for d in dsets],
             local_path=[None for d in dsets],
-            eid=[eid for d in dsets],  # [ses_info['url'][-36:] for d in dsets],
+            eid=[eid for _ in dsets],  # [ses_info['url'][-36:] for d in dsets],
             url=[d['data_url'] for d in dsets],
-            data=[None for d in dsets],
+            data=[None for _ in dsets],
             hash=[d['hash'] for d in dsets],
             file_size=[d['file_size'] for d in dsets]
         )
@@ -75,6 +76,25 @@ class SessionDataInfo:
     def from_session_details(ses_info, **kwargs):
         return _session_details_to_dataclasses(ses_info, **kwargs)
 
+    @staticmethod
+    def from_pandas(df, cache_dir):
+        if df.size == 0:
+            return SessionDataInfo()
+        fcn_local_path = lambda rec: Path(cache_dir).joinpath(  # noqa
+            rec['lab'], 'Subjects', rec['subject'], rec['start_time'].isoformat()[:10],
+            str(rec['number']).zfill(3), rec['collection'], rec['name'])
+        nrecs = df.shape[0]
+        return SessionDataInfo(
+            dataset_type=df.dataset_type.to_list(),
+            dataset_id=list(parquet.np2str(df[['id_0', 'id_1']])),
+            local_path=df.apply(fcn_local_path, axis=1).to_list(),
+            eid=list(parquet.np2str(df[['eid_0', 'eid_1']])),
+            url=[None for _ in range(nrecs)],
+            data=[None for _ in range(nrecs)],
+            hash=df.hash.to_list(),
+            file_size=df.file_size.to_list()
+        )
+
 
 @singledispatch
 def _session_details_to_dataclasses(ses_info, **kwargs):
