@@ -1,16 +1,18 @@
-import logging
-import os
-from pathlib import Path
 import json
+import logging
+import math
+import os
 import re
 import urllib.request
-import requests
 from collections.abc import Mapping
-import math
+from pathlib import Path
+import hashlib
+
+import requests
 
 from ibllib.misc import pprint, print_progress
 
-logger_ = logging.getLogger('ibllib')
+_logger = logging.getLogger('ibllib')
 
 
 class _PaginatedResponse(Mapping):
@@ -67,7 +69,7 @@ def http_download_file_list(links_to_file_list, **kwargs):
 
 
 def http_download_file(full_link_to_file, *, clobber=False, offline=False,
-                       username='', password='', cache_dir=''):
+                       username='', password='', cache_dir='', return_md5=False):
     """
     :param full_link_to_file: http link to the file.
     :type full_link_to_file: str
@@ -83,6 +85,7 @@ def http_download_file(full_link_to_file, *, clobber=False, offline=False,
 
     :return: (str) a list of the local full path of the downloaded files.
     """
+    from ibllib.io import hashfile
     if not full_link_to_file:
         return ''
 
@@ -91,13 +94,13 @@ def http_download_file(full_link_to_file, *, clobber=False, offline=False,
         cache_dir = str(Path.home().joinpath("Downloads"))
 
     # This is the local file name
-    file_name = cache_dir + os.sep + os.path.basename(full_link_to_file)
+    file_name = str(cache_dir) + os.sep + os.path.basename(full_link_to_file)
 
     # do not overwrite an existing file unless specified
     if not clobber and os.path.exists(file_name):
-        return file_name
+        return (file_name, hashfile.md5(file_name)) if return_md5 else file_name
     elif offline:
-        return file_name
+        return (file_name, hashfile.md5(file_name)) if return_md5 else file_name
 
     # This should be the base url you wanted to access.
     baseurl = os.path.split(str(full_link_to_file))[0]
@@ -115,12 +118,19 @@ def http_download_file(full_link_to_file, *, clobber=False, offline=False,
     urllib.request.install_opener(opener)
 
     # Open the url and get the length
-    u = urllib.request.urlopen(full_link_to_file)
+    try:
+        u = urllib.request.urlopen(full_link_to_file)
+    except urllib.error.HTTPError as e:
+        _logger.error(f"{str(e)} {full_link_to_file}")
+        raise e
+
     file_size = int(u.getheader('Content-length'))
 
     print(f"Downloading: {file_name} Bytes: {file_size}")
     file_size_dl = 0
     block_sz = 8192 * 64 * 8
+
+    md5 = hashlib.md5()
     f = open(file_name, 'wb')
     while True:
         buffer = u.read(block_sz)
@@ -128,10 +138,12 @@ def http_download_file(full_link_to_file, *, clobber=False, offline=False,
             break
         file_size_dl += len(buffer)
         f.write(buffer)
+        if return_md5:
+            md5.update(buffer)
         print_progress(file_size_dl, file_size, prefix='', suffix='')
     f.close()
 
-    return file_name
+    return (file_name, md5.hexdigest()) if return_md5 else file_name
 
 
 def file_record_to_url(file_records, urls=[]):
@@ -169,12 +181,32 @@ def dataset_record_to_url(dataset_record):
     return urls
 
 
-class AlyxClient:
+class UniqueSingletons(type):
+    _instances: list = []
+
+    def __call__(cls, *args, **kwargs):
+        # print('args', args, '\nkwargs', kwargs)
+        for inst in UniqueSingletons._instances:
+            if cls in inst and inst.get(cls, None).get('args') == (args, kwargs):
+                return inst[cls].get('instance')
+
+        new_instance = super(UniqueSingletons, cls).__call__(*args, **kwargs)
+        # Optional rerun of constructor
+        # new_instance.__init__(*args, **kwargs)
+        new_instance_record = {
+            cls: {'args': (args, kwargs), 'instance': new_instance}
+        }
+        UniqueSingletons._instances.append(new_instance_record)
+
+        return new_instance
+
+
+class AlyxClient(metaclass=UniqueSingletons):
     """
     Class that implements simple GET/POST wrappers for the Alyx REST API
     http://alyx.readthedocs.io/en/latest/api.html
     """
-    _token = ''
+    _token = ''  # These class params are not being used!!
     _headers = ''
     _rest_schemes = ''
 
@@ -196,24 +228,25 @@ class AlyxClient:
         self._rest_schemes = self.get('/docs')
         # the mixed accept application may cause errors sometimes, only necessary for the docs
         self._headers['Accept'] = 'application/json'
+        self._obj_id = id(self)
 
     def _generic_request(self, reqfunction, rest_query, data=None):
         # if the data is a dictionary, it has to be converted to json text
-        if isinstance(data, dict):
+        if isinstance(data, dict) or isinstance(data, list):
             data = json.dumps(data)
         # makes sure the base url is the one from the instance
         rest_query = rest_query.replace(self._base_url, '')
         if not rest_query.startswith('/'):
             rest_query = '/' + rest_query
-        logger_.debug(self._base_url + rest_query)
+        _logger.debug(self._base_url + rest_query)
         r = reqfunction(self._base_url + rest_query, stream=True, headers=self._headers, data=data)
         if r and r.status_code in (200, 201):
             return json.loads(r.text)
         elif r and r.status_code == 204:
             return
         else:
-            logger_.error(self._base_url + rest_query)
-            logger_.error(r.text)
+            _logger.error(self._base_url + rest_query)
+            _logger.error(r.text)
             raise(requests.HTTPError(r))
 
     def authenticate(self, username='', password='', base_url=''):
@@ -231,9 +264,10 @@ class AlyxClient:
         self._base_url = base_url
         rep = requests.post(base_url + '/auth-token',
                             data=dict(username=username, password=password))
-        self._token = rep.json()
+        # Assign token or raise exception on internal server error
+        self._token = rep.json() if rep.ok else rep.raise_for_status()
         if not (list(self._token.keys()) == ['token']):
-            logger_.error(rep)
+            _logger.error(rep)
             raise Exception('Alyx authentication error. Check your credentials')
         self._headers = {
             'Authorization': 'Token {}'.format(list(self._token.values())[0]),
@@ -267,6 +301,7 @@ class AlyxClient:
         :return: (dict/list) json interpreted dictionary from response
         """
         rep = self._generic_request(requests.get, rest_query)
+        _logger.debug(rest_query)
         if isinstance(rep, dict) and list(rep.keys()) == ['count', 'next', 'previous', 'results']:
             if len(rep['results']) < rep['count']:
                 rep = _PaginatedResponse(self, rep)
@@ -334,7 +369,7 @@ class AlyxClient:
             alyx.client.rest('subjects', 'create', data=sub_dict)
             alyx.client.rest('subjects', 'read', id='nickname')
             alyx.client.rest('subjects', 'update', id='nickname', data=sub_dict)
-            alyx.client.rest('subjects', 'partial_update', id='nickname', data=sub_ict)
+            alyx.client.rest('subjects', 'partial_update', id='nickname', data=sub_dict)
             alyx.client.rest('subjects', 'delete', id='nickname')
 
         :param url: endpoint name
@@ -373,7 +408,7 @@ class AlyxClient:
                              '\n       ' + '\n       '.join(endpoint_scheme.keys()))
         # the actions below require an id in the URL, warn and help the user
         if action in ['read', 'update', 'partial_update', 'delete'] and not id:
-            logger_.warning('REST action "' + action + '" requires an ID in the URL: ' +
+            _logger.warning('REST action "' + action + '" requires an ID in the URL: ' +
                             endpoint_scheme[action]['url'])
             return
         # the actions below require a data dictionary, warn and help the user with fields list
@@ -381,7 +416,7 @@ class AlyxClient:
             pprint(endpoint_scheme[action]['fields'])
             for act in endpoint_scheme[action]['fields']:
                 print("'" + act['name'] + "': ...,")
-            logger_.warning('REST action "' + action + '" requires a data dict with above keys')
+            _logger.warning('REST action "' + action + '" requires a data dict with above keys')
             return
 
         if action == 'list':
@@ -417,3 +452,140 @@ class AlyxClient:
         elif action == 'update':
             assert(endpoint_scheme[action]['action'] == 'put')
             return self.put('/' + endpoint + '/' + id.split('/')[-1], data=data)
+
+    # JSON field interface convenience methods
+    def _check_inputs(self, endpoint: str) -> None:
+        # make sure the queryied endpoint exists, if not throw an informative error
+        if endpoint not in self._rest_schemes.keys():
+            av = [k for k in self._rest_schemes.keys() if not k.startswith('_') and k]
+            raise ValueError('REST endpoint "' + endpoint + '" does not exist. Available ' +
+                             'endpoints are \n       ' + '\n       '.join(av))
+        return
+
+    def json_field_write(
+        self,
+        endpoint: str = None,
+        uuid: str = None,
+        field_name: str = None,
+        data: dict = None
+    ) -> dict:
+        """json_field_write [summary]
+        Write data to WILL NOT CHECK IF DATA EXISTS
+        NOTE: Destructive write!
+
+        :param endpoint: Valid alyx endpoint, defaults to None
+        :type endpoint: str, optional
+        :param uuid: uuid or lookup name for endpoint
+        :type uuid: str, optional
+        :param field_name: Valid json field name, defaults to None
+        :type field_name: str, optional
+        :param data: data to write to json field, defaults to None
+        :type data: dict, optional
+        :return: Written data dict
+        :rtype: dict
+        """
+        self._check_inputs(endpoint)
+        # Prepare data to patch
+        patch_dict = {field_name: data}
+        # Upload new extended_qc to session
+        ret = self.rest(endpoint, "partial_update", id=uuid, data=patch_dict)
+        return ret[field_name]
+
+    def json_field_update(
+        self,
+        endpoint: str = None,
+        uuid: str = None,
+        field_name: str = 'json',
+        data: dict = None
+    ) -> dict:
+        """json_field_update
+        Non destructive update of json field of endpoint for object
+        Will update the field_name of the object with pk = uuid of given endpoint
+        If data has keys with the same name of existing keys it will squash the old
+        values (uses the dict.update() method)
+
+        Example:
+        one.alyx.json_field_update("sessions", "eid_str", "extended_qc" {"key": value})
+
+        :param endpoint: endpoint to hit
+        :type endpoint: str
+        :param uuid: uuid or lookup name of object
+        :type uuid: str
+        :param field_name: name of the json field
+        :type field_name: str
+        :param data: dictionary with fields to be updated
+        :type data: dict
+        :return: new patched json field contents
+        :rtype: dict
+        """
+        self._check_inputs(endpoint)
+        # Load current json field contents
+        current = self.rest(endpoint, "read", id=uuid)[field_name]
+        if current is None:
+            current = {}
+
+        if not isinstance(current, dict):
+            _logger.warning(
+                f"Current json field {field_name} does not contains a dict, aborting update"
+            )
+            return current
+
+        # Patch current dict with new data
+        current.update(data)
+        # Prepare data to patch
+        patch_dict = {field_name: current}
+        # Upload new extended_qc to session
+        ret = self.rest(endpoint, "partial_update", id=uuid, data=patch_dict)
+        return ret[field_name]
+
+    def json_field_remove_key(
+        self,
+        endpoint: str = None,
+        uuid: str = None,
+        field_name: str = 'json',
+        key: str = None
+    ) -> dict:
+        """json_field_remove_key
+        Will remove inputted key from json field dict and reupload it to Alyx.
+        Needs endpoint, uuid and json field name
+
+        :param endpoint: endpoint to hit, defaults to None
+        :type endpoint: str, optional
+        :param uuid: uuid or lookup name for endpoint
+        :type uuid: str, optional
+        :param field_name: json field name of object, defaults to None
+        :type field_name: str, optional
+        :param key: key name of dictionary inside object, defaults to None
+        :type key: str, optional
+        :return: returns new content of json field
+        :rtype: dict
+        """
+        self._check_inputs(endpoint)
+        current = self.rest(endpoint, "read", id=uuid)[field_name]
+        # If no contents, cannot remove key, return
+        if current is None:
+            return current
+        # if contents are not dict, cannot remove key, return contents
+        if isinstance(current, str):
+            _logger.warning(f"Cannot remove key {key} content of json field is of type str")
+            return current
+        # If key not present in contents of json field cannot remove key, return contents
+        if current.get(key, None) is None:
+            _logger.warning(
+                f"{key}: Key not found in endpoint {endpoint} field {field_name}"
+            )
+            return current
+        _logger.info(f"Removing key from dict: '{key}'")
+        current.pop(key)
+        # Re-write contents without removed key
+        written = self.json_field_write(
+            endpoint=endpoint, uuid=uuid, field_name=field_name, data=current
+        )
+        return written
+
+    def json_field_delete(
+        self, endpoint: str = None, uuid: str = None, field_name: str = None
+    ) -> None:
+        self._check_inputs(endpoint)
+        _ = self.rest(endpoint, "partial_update", id=uuid, data={field_name: None})
+        return _[field_name]
