@@ -1,9 +1,8 @@
 import unittest
-
+from pathlib import Path
 import numpy as np
 
 import ibllib.io.extractors.ephys_fpga as ephys_fpga
-import ibllib.io.extractors.ephys_trials as ephys_trials
 import ibllib.io.raw_data_loaders as raw
 
 
@@ -43,8 +42,8 @@ class TestEphysSyncExtraction(unittest.TestCase):
                                 179.41063333, 179.41073333, 181.70343333, 181.85343333,
                                 183.12896667, 183.12906667])
 
-        t_trial_start, t_valve_open, _ = ephys_fpga._bpod_events_extraction(bpod_times_,
-                                                                            bpod_fronts_)
+        t_trial_start, t_valve_open, _ = ephys_fpga._assign_events_bpod(bpod_times_,
+                                                                        bpod_fronts_)
         self.assertTrue(np.all(np.isclose(t_trial_start, t_trial_start_)))
         self.assertTrue(np.all(np.isclose(t_valve_open, t_valve_open_)))
 
@@ -77,12 +76,20 @@ class TestEphysSyncExtraction(unittest.TestCase):
         self.assertTrue(np.allclose(desired_out, t_event_nans, equal_nan=True, atol=0, rtol=0))
 
         # same test above but this time take the first index instead of last
-        t_trial_start = np.array([0, 10, 20, 30, 40])
-        t_event = np.array([-1, 2, 4, 12, 35, 42])
         t_event_nans = ephys_fpga._assign_events_to_trial(t_trial_start, t_event, take='first')
         desired_out = np.array([2, 12., np.nan, 35, 42])
         self.assertTrue(np.allclose(desired_out, t_event_nans, equal_nan=True, atol=0, rtol=0))
 
+        # take second to last
+        t_trial_start = np.array([0, 10, 20, 30, 40])
+        t_event = np.array([2, 4, 12, 13, 14, 21, 32, 33, 34, 35, 42])
+        t_event_nans = ephys_fpga._assign_events_to_trial(t_trial_start, t_event, take=-2)
+        desired_out = np.array([2, 13, np.nan, 34, np.nan])
+        self.assertTrue(np.allclose(desired_out, t_event_nans, equal_nan=True, atol=0, rtol=0))
+        t_event_nans = ephys_fpga._assign_events_to_trial(t_trial_start, t_event, take=1)
+        desired_out = np.array([4, 13, np.nan, 33, np.nan])
+        self.assertTrue(np.allclose(desired_out, t_event_nans, equal_nan=True, atol=0, rtol=0))
+
     def test_wheel_trace_from_sync(self):
         pos_ = - np.array([-1, 0, -1, -2, -1, -2]) * (np.pi / ephys_fpga.WHEEL_TICKS)
         ta = np.array([1, 2, 3, 4, 5, 6])
@@ -101,25 +108,20 @@ class TestEphysSyncExtraction(unittest.TestCase):
 
 class TestEphysBehaviorExtraction(unittest.TestCase):
     def setUp(self):
-        self.session_path = "tests/ibllib/extractors/data/session_ephys"
+        self.session_path = Path(__file__).parent.joinpath('data', 'session_ephys')
 
     def test_get_probabilityLeft(self):
         data = raw.load_data(self.session_path)
         settings = raw.load_settings(self.session_path)
-        pLeft0 = ephys_trials.get_probabilityLeft(
-            self.session_path, save=False, data=data, settings=False
-        )
+        pLeft0, _, _ = ephys_fpga.ProbaContrasts(
+            self.session_path).extract(bpod_trials=data, settings=settings)[0]
         self.assertTrue(len(pLeft0) == len(data))
         # Test if only generative prob values in data
         self.assertTrue(all([x in [0.2, 0.5, 0.8] for x in np.unique(pLeft0)]))
         # Test if settings file has empty LEN_DATA result is same
         settings.update({"LEN_BLOCKS": None})
-        pLeft1 = ephys_trials.get_probabilityLeft(
-            self.session_path, save=False, data=False, settings=settings
-        )
+        pLeft1, _, _ = ephys_fpga.ProbaContrasts(
+            self.session_path).extract(bpod_trials=data, settings=settings)[0]
         self.assertTrue(all(pLeft0 == pLeft1))
         # Test if only generative prob values in data
         self.assertTrue(all([x in [0.2, 0.5, 0.8] for x in np.unique(pLeft1)]))
-
-    def tearDown(self):
-        pass
