@@ -1,9 +1,48 @@
 import unittest
 import numpy as np
-import numpy.matlib as mat
+import scipy.signal
 
 import ibllib.dsp.fourier as ft
-from ibllib.dsp import WindowGenerator, rms, rises, falls, fronts, smooth
+from ibllib.dsp import WindowGenerator, rms, rises, falls, fronts, smooth, shift, fit_phase,\
+    fcn_cosine
+
+
+class TestDspMisc(unittest.TestCase):
+
+    def test_dsp_cosine_func(self):
+        x = np.linspace(0, 40)
+        fcn = fcn_cosine(bounds=[20, 30])
+        y = fcn(x)
+        self.assertTrue(y[0] == 0 and y[-1] == 1 and np.all(np.diff(y) >= 0))
+
+
+class TestPhaseRegression(unittest.TestCase):
+
+    def test_fit_phase1d(self):
+        w = np.zeros(500)
+        w[1] = 1
+        self.assertTrue(np.isclose(fit_phase(w, .002), .002))
+
+    def test_fit_phase2d(self):
+        w = np.zeros((500, 2))
+        w[1, 0], w[2, 1] = (1, 1)
+        self.assertTrue(np.all(np.isclose(fit_phase(w, .002, axis=0), np.array([.002, .004]))))
+        self.assertTrue(np.all(np.isclose(fit_phase(w.transpose(), .002), np.array([.002, .004]))))
+
+
+class TestShift(unittest.TestCase):
+
+    def test_shift_1d(self):
+        ns = 500
+        w = scipy.signal.ricker(ns, 10)
+        self.assertTrue(np.all(np.isclose(shift(w, 1), np.roll(w, 1))))
+
+    def test_shift_2d(self):
+        ns = 500
+        w = scipy.signal.ricker(ns, 10)
+        w = np.tile(w, (100, 1)).transpose()
+        self.assertTrue(np.all(np.isclose(shift(w, 1, axis=0), np.roll(w, 1, axis=0))))
+        self.assertTrue(np.all(np.isclose(shift(w, 1, axis=1), np.roll(w, 1, axis=1))))
 
 
 class TestSmooth(unittest.TestCase):
@@ -18,6 +57,25 @@ class TestSmooth(unittest.TestCase):
 
 class TestFFT(unittest.TestCase):
 
+    def test_spectral_convolution(self):
+        sig = np.random.randn(20, 500)
+        w = np.hanning(25)
+        c = ft.convolve(sig, w)
+        s = np.convolve(sig[0, :], w)
+        self.assertTrue(np.all(np.isclose(s, c[0, :-1])))
+
+        c = ft.convolve(sig, w, mode='same')
+        s = np.convolve(sig[0, :], w, mode='same')
+        self.assertTrue(np.all(np.isclose(c[0, :], s)))
+
+        c = ft.convolve(sig, w[:-1], mode='same')
+        s = np.convolve(sig[0, :], w[:-1], mode='same')
+        self.assertTrue(np.all(np.isclose(c[0, :], s)))
+
+    def test_nech_optim(self):
+        self.assertTrue(ft.ns_optim_fft(2048) == 2048)
+        self.assertTrue(ft.ns_optim_fft(65532) == 65536)
+
     def test_imports(self):
         import ibllib.dsp as dsp
         self.assertTrue(len([dsp.lp,
@@ -35,7 +93,7 @@ class TestFFT(unittest.TestCase):
         self.assertTrue(np.all(ft.freduce(fs) == fs[:-2]))
 
         # test 2D arrays along both dimensions
-        fs = mat.repmat(ft.fscale(500, 0.001), 4, 1)
+        fs = np.tile(ft.fscale(500, 0.001), (4, 1))
         self.assertTrue(ft.freduce(fs).shape == (4, 251))
         self.assertTrue(ft.freduce(np.transpose(fs), axis=0).shape == (251, 4))
 
@@ -61,7 +119,7 @@ class TestFFT(unittest.TestCase):
         R = np.real(np.fft.ifft(ft.fexpand(X, 12)))
         self.assertTrue(np.all((res - R) < 1e-6))
         # test with 2 dimensional input along first dimension
-        fs = np.transpose(mat.repmat(ft.fscale(500, 0.001, one_sided=True), 4, 1))
+        fs = np.transpose(np.tile(ft.fscale(500, 0.001, one_sided=True), (4, 1)))
         self.assertTrue(ft.fexpand(fs, 500, axis=0).shape == (500, 4))
 
     def test_fscale(self):
@@ -80,17 +138,41 @@ class TestFFT(unittest.TestCase):
         out1 = ft.lp(ts1, 1, [.1, .2])
         self.assertTrue(np.mean(ts1 - out1) < 0.001)
         # test 2D case along the last dimension
-        ts = mat.repmat(ts1, 11, 1)
+        ts = np.tile(ts1, (11, 1))
         out = ft.lp(ts, 1, [.1, .2])
         self.assertTrue(np.allclose(out, out1))
         # test 2D case along the first dimension
-        ts = mat.repmat(ts1[:, np.newaxis], 1, 11)
+        ts = np.tile(ts1[:, np.newaxis], (1, 11))
         out = ft.lp(ts, 1, [.1, .2], axis=0)
         self.assertTrue(np.allclose(np.transpose(out), out1))
         # test 1D time serie: subtracting lp filter removes DC
         out2 = ft.hp(ts1, 1, [.1, .2])
         self.assertTrue(np.allclose(out1, ts1 - out2))
 
+    def test_dft(self):
+        # test 1D complex
+        x = np.array([1, 2 - 1j, -1j, -1 + 2j])
+        X = ft.dft(x)
+        assert np.all(np.isclose(X, np.fft.fft(x)))
+        # test 1D real
+        x = np.random.randn(7)
+        X = ft.dft(x)
+        assert np.all(np.isclose(X, np.fft.rfft(x)))
+        # test along the 3 dimensions of a 3D array
+        x = np.random.rand(10, 11, 12)
+        for axis in np.arange(3):
+            X_ = np.fft.rfft(x, axis=axis)
+            assert np.all(np.isclose(X_, ft.dft(x, axis=axis)))
+        # test 2D irregular grid
+        _n0, _n1, nt = (10, 11, 30)
+        x = np.random.rand(_n0 * _n1, nt)
+        X_ = np.fft.fft(np.fft.fft(x.reshape(_n0, _n1, nt), axis=0), axis=1)
+        r, c = [v.flatten() for v in np.meshgrid(np.arange(
+            _n0) / _n0, np.arange(_n1) / _n1, indexing='ij')]
+        nk, nl = (_n0, _n1)
+        X = ft.dft2(x, r, c, nk, nl)
+        assert np.all(np.isclose(X, X_))
+
 
 class TestWindowGenerator(unittest.TestCase):
 
