@@ -2,23 +2,47 @@ import unittest
 import numpy as np
 import requests
 from pathlib import Path
+import tempfile
+import shutil
 
-import ibllib.io.hashfile
+import ibllib.io.hashfile as hashfile
 from alf.io import remove_uuid_file
-from oneibl.one import ONE
+from oneibl.one import ONE, SessionDataInfo
 
 one = ONE(base_url='https://test.alyx.internationalbrainlab.org', username='test_user',
           password='TapetesBloc18')
 
 
+class TestOneOffline(unittest.TestCase):
+
+    def test_one_offline(self) -> None:
+        # init: create a temp directory and copy the fixtures
+        init_cache_file = Path(__file__).parent.joinpath('fixtures', '.one_cache.parquet')
+        with tempfile.TemporaryDirectory() as td:
+            cache_dir = Path(td)
+            shutil.copyfile(init_cache_file, cache_dir.joinpath(init_cache_file.name))
+
+            # test the constructor
+            self.assertTrue(one._cache.shape[1] == 14)
+
+            # test the load with download false so it returns only file paths
+            eid = 'cf264653-2deb-44cb-aa84-89b82507028a'
+            one.list(eid)
+            dtypes = ['_spikeglx_sync.channels',
+                      '_spikeglx_sync.polarities',
+                      '_spikeglx_sync.times',
+                      '_iblrig_taskData.raw',
+                      '_iblrig_taskSettings.raw',
+                      'ephysData.raw.meta',
+                      'camera.times',
+                      'ephysData.raw.wiring']
+            one.load(eid, dataset_types=dtypes, dclass_output=False,
+                     download_only=True, offline=False)
+
+
 class TestSearch(unittest.TestCase):
 
-    def setUp(self):
-        # Init connection to the database
-        self.One = one
-
     def test_search_simple(self):
-        one = self.One
         # Test users
         usr = ['olivier', 'nbonacchi']
         sl, sd = one.search(users=usr, details=True)
@@ -56,18 +80,11 @@ class TestList(unittest.TestCase):
 
     def test_list(self):
         # tests with a single input and a list input
-        EIDs = [self.eid,
-                [self.eid, self.eid2]]
-        for eid in EIDs:
-            dt = one.list(eid)  # returns dataset-type
-            dt = one.list(eid, details=True)
-            dt = one.list(eid, keyword='dataset-type')  # returns list
-            dt = one.list(eid, keyword='dataset-type', details=True)  # returns SessionDataInfo
-            for key in ('subject', 'users', 'lab', 'type', 'start_time', 'end_time'):
-                dt = one.list(eid, keyword=key)  # returns dataset-type
-                print(key, ': ', dt)
-            ses = one.list(eid, keyword='all')
-            usr, ses = one.list(eid, keyword='users', details=True)
+        eid = self.eid
+        dt = one.list(eid)  # returns dataset-type
+        self.assertTrue(isinstance(dt, list))
+        dt = one.list(eid, details=True)
+        self.assertTrue(isinstance(dt, SessionDataInfo))
 
     def test_list_error(self):
         a = 0
@@ -150,14 +167,14 @@ class TestLoad(unittest.TestCase):
         d = one.load(eid, dataset_types=dataset_types, dclass_output=True)
         ind = int(np.where(np.array(d.dataset_type) == 'clusters.channels')[0])
         self.assertTrue(np.all(d.data[ind] == t))
-        # Now load with another dset inbetween that doesn't exist
+        # Now load with another dset in between that doesn't exist
         t_, cr_, cl_ = one.load(eid, dataset_types=['clusters.channels', 'turlu',
                                                     'clusters.probes'])
         self.assertTrue(np.all(t == t_))
         self.assertTrue(np.all(cl == cl_))
         self.assertTrue(cr_ is None)
         # Now try in offline mode where the file already exists
-        t_ = one.load(eid, dataset_types=['clusters.channels'], offline=True)
+        t_ = one.load(eid, dataset_types=['clusters.channels'])
         self.assertTrue(np.all(t == t_))
 
         # load_dataset()
@@ -198,17 +215,13 @@ class TestLoad(unittest.TestCase):
         eid = 'aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee'
         self.assertRaises(requests.HTTPError, one.load, eid)
 
-    def test_load_offline(self):
-        a = one.load(self.eid, dataset_types='_ibl_lickPiezo.raw.npy', offline=True)
-        self.assertTrue(a == [None])
-
-    def test_load_newversion(self):
+    def test_download_hash(self):
         eid = self.eid
         # get the original file from the server
         file = one.load(eid, dataset_types=['channels.localCoordinates'], download_only=True,
                         clobber=True)[0]
         fsize = file.stat().st_size
-        hash = ibllib.io.hashfile.md5(file)
+        hash = hashfile.md5(file)
         data_server = np.load(file)
         # overwrite the local file
         np.save(file, np.zeros([25, 0]))
@@ -224,6 +237,15 @@ class TestLoad(unittest.TestCase):
         data = one.load(eid, dataset_types=['channels.localCoordinates'])[0]
         self.assertTrue(data.shape == data_server.shape)
         self.assertTrue(np.all(np.equal(data, data_server)))
+        # here we corrupt the md5 hash on the database, the file will get downloaded again,
+        # but on checking the file one.load should have labeled the json field for database
+        # maintenance
+        one.alyx.rest('datasets', 'partial_update', id=dset[0]['url'][-36:],
+                      data={'file_size': fsize, 'hash': "5d1d13589934440a9947c2477b2e61ea"})
+        one.load(eid, dataset_types=['channels.localCoordinates'])[0]
+        fr = one.alyx.rest('files', 'list', django=f"dataset,{dset[0]['url'][-36:]},"
+                                                   f"data_repository__globus_is_personal,False")
+        self.assertTrue(fr[0]['json'] == {'mismatch_hash': True})
 
 
 class TestMisc(unittest.TestCase):
@@ -243,51 +265,5 @@ class TestMisc(unittest.TestCase):
         self.assertEqual(_validate_date_range(val), val)
 
 
-class TestPathsToEidAndBack(unittest.TestCase):
-
-    def setUp(self):
-        # Init connection to the database
-        self.eids = ['cf264653-2deb-44cb-aa84-89b82507028a',
-                     '4e0b3320-47b7-416e-b842-c34dc9004cf8',
-                     'a9d89baf-9905-470c-8565-859ff212c7be',
-                     'aaf101c3-2581-450a-8abd-ddb8f557a5ad',
-                     ]
-        self.partial_eid_paths = [
-            None,
-            None,
-            'FlatIron/mainenlab/Subjects/ZM_1743/2019-06-04/001',
-            'FlatIron/cortexlab/Subjects/KS005/2019-04-04/004',
-        ]
-
-    def test_path_from_eid(self):
-        # Test if eid's produce correct output
-        for e, p in zip(self.eids, self.partial_eid_paths):
-            self.assertTrue(str(p) in str(one.path_from_eid(e)))
-        # Test if list input produces valid list output
-        list_output = one.path_from_eid(self.eids)
-        self.assertTrue(isinstance(list_output, list))
-        self.assertTrue(
-            all([str(p) in str(o) for p, o in zip(self.partial_eid_paths, list_output)])
-        )
-
-    def test_eid_from_path(self):
-        # test if paths produce expected eid's
-        paths = self.partial_eid_paths[-2:]
-        paths.append('FlatIron/mainenlab/Subjects/ZM_1743/2019-06-04/001/bla.ble')
-        paths.append('some/other/root/FlatIron/cortexlab/'
-                     'Subjects/KS005/2019-04-04/004/bli/blo.blu')
-        eids = self.eids[-2:]
-        eids.append('a9d89baf-9905-470c-8565-859ff212c7be')
-        eids.append('aaf101c3-2581-450a-8abd-ddb8f557a5ad')
-        for p, e in zip(paths, eids):
-            self.assertTrue(e == str(one.eid_from_path(p)))
-        # Test if list input produces correct list output
-        list_output = one.eid_from_path(paths)
-        self.assertTrue(isinstance(list_output, list))
-        self.assertTrue(
-            all([e == o for e, o in zip(eids, list_output)])
-        )
-
-
 if __name__ == '__main__':
     unittest.main(exit=False)
